from pathlib import Path

import click
from flask import Flask

from .blueprints import beans


@click.command()
@click.option('--host', default='127.0.0.1', help='default: 127.0.0.1')
@click.option('--port', default=3000, help='default: 3000')
@click.option('--debug', default=False, is_flag=True, help='Enables debug mode')
@click.argument('contents', type=click.Path(exists=True))
def main(host, port, debug, contents):
    app = Flask(__name__)
    app.config['BEANS'] = Path(contents).absolute()
    app.register_blueprint(beans)
    app.run(host=host, port=port, debug=debug)



import json
import logging

import jsonschema
from flask import request

from .exceptions import (
    BeanValidationError,
    ContentNotFoundError,
    InvalidContent,
)


log = logging.getLogger(__name__)


class ContentResolver(object):

    BEAN_SCHEMA = {
        "type": "object",
        "properties": {
            "_bean": {"type": "boolean"},
            "responses": {
                "type": "array",
                "minItems": 1,
                "items": {
                    "type": "object",
                    "properties": {
                        "contents": {"type": "object"},
                        "params": {"type": "object"},
                        "status": {"type": "number"}
                    },
                    "required": ["contents"]
                }
            },
        },
        "required": [
            '_bean',
            'responses'
        ]
    }

    RESPONSE_CODES = {
        'get': 200,
        'post': 201,
        'delete': 204,
        'put': 200,
        'patch': 200
    }

    def __init__(self, base, url, method):
        self.base = base
        self.url = url.replace('//', '/')
        self.method = method

    def build_path(self):
        if self.url.endswith('/') or self.url == '':
            self.path = f'{self.url}index.{self.method}.json'
        else:
            self.path = f'{self.url}.{self.method}.json'
        return self.path

    def resolve_path(self):
        self.full_path = self.base / self.path
        if not self.full_path.exists():
            log.warn(f'Cannot locate content for {self.url}, {self.full_path} not found')
            raise ContentNotFoundError()
        return self.full_path

    def is_bean(self, content):
        return isinstance(content, dict) and content.get('_bean') is True

    def load_content(self):
        with self.full_path.open() as f:
            try:
                return json.load(f)
            except ValueError as e:
                log.error(f'Unable to parse {self.full_path}: {e}')
                raise InvalidContent()

    def validate_bean(self, bean):
        try:
            jsonschema.validate(bean, self.BEAN_SCHEMA)
        except jsonschema.ValidationError as e:
            log.error(f"{self.full_path} contains an invalid bean: {e}")
            raise BeanValidationError()

    def matches_params(self, params):
        for param, value in params.items():
            if request.args.get(param) != str(value):
                return False
        return True

    def resolve_bean(self, bean):
        responses = bean['responses']
        response = None
        for r in responses:
            if self.matches_params(r.get('params', {})):
                response = r
                break
        if response is None:
            log.warn(f"No response matched for {self.url}, defaulting to first")
            response = responses[0]
        return (
            response.get('contents', {}),
            response.get('status', self.default_response_code())
        )

    def default_response_code(self):
        return self.RESPONSE_CODES.get(self.method, 200)

    @property
    def response(self):
        self.build_path()
        self.resolve_path()
        content = self.load_content()
        if self.is_bean(content):
            log.info('Content is a bean, resolving')
            self.validate_bean(content)
            return self.resolve_bean(content)
        return content, self.default_response_code()



import io
import os

from setuptools import setup


NAME = 'bakedbeans'

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

about = {}
with open(os.path.join(here, NAME, '__version__.py')) as f:
    exec(f.read(), about)

setup(
    name='bakedbeans',
    version=about['__version__'],
    author='Jonathan Moss',
    author_email='jmoss@commoncode.io',
    url='https://github.com/a-musing-moose/bakedbeans',
    license='MIT',
    packages=[NAME],
    install_requires=[
        'click~=6.7',
        'Flask~=0.12.2',
        'jsonschema~=2.6.0'
    ],
    description='Canned response HTTP server',
    long_description=long_description,
    classifiers=(
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'Programming Language :: Python :: 3.6',
    ),
    entry_points={
        'console_scripts': ['baked=bakedbeans.cli:main'],
    }
)



from flask import Blueprint, current_app, jsonify, request

from .contents import ContentResolver
from .exceptions import (
    BeanValidationError,
    ContentNotFoundError,
    InvalidContent,
)


beans = Blueprint('name', __name__)


@beans.route('/', defaults={'path': ''},
             methods=['GET', 'POST', "DELETE", "PUT", "PATCH"])
@beans.route('/<path:path>', methods=['GET', 'POST', "DELETE", "PUT", "PATCH"])
def catch_all(path: str):
    try:
        body, status = ContentResolver(
            base=current_app.config['BEANS'],
            url=path,
            method=request.method.lower()
        ).response
    except ContentNotFoundError:
        return jsonify({'error': f'content not found {path}'}), 404
    except InvalidContent:
        return jsonify({'error': 'content invalid'}), 500
    except BeanValidationError:
        return jsonify({'error': 'This is one mouldy bean'}), 500
    return jsonify(body), status



class BakedBeansError(Exception):
    pass


class ContentNotFoundError(BakedBeansError):
    pass


class InvalidContent(BakedBeansError):
    pass


class BeanValidationError(BakedBeansError):
    pass
